package com.gic.haoban.manage.service.service.out.impl;


import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.haoban.app.customer.enums.GicQywxSyncTypeEnum;
import com.gic.haoban.app.customer.enums.QywxSyncTaskTypeEnum;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.base.api.common.ServiceResponse;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.manage.api.dto.ExternalClerkRelatedDTO;
import com.gic.haoban.manage.api.dto.ExternalClerkRelatedShortInfoDTO;
import com.gic.haoban.manage.api.service.ExternalClerkRelatedApiService;
import com.gic.haoban.manage.service.dao.mapper.MemberUnionidRelatedMapper;
import com.gic.haoban.manage.service.dao.mapper.TabHaobanExternalClerkRelatedMapper;
import com.gic.haoban.manage.service.dao.mapper.TabHaobanStaffClerkRelationMapper;
import com.gic.haoban.manage.service.entity.MemberUnionidRelated;
import com.gic.haoban.manage.service.entity.TabHaobanExternalClerkRelated;
import com.gic.haoban.manage.service.entity.TabHaobanStaffClerkRelation;
import com.gic.haoban.manage.service.pojo.QywxTagSyncInfoPojo;
import com.gic.mq.sdk.GicMQClient;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.Date;
import java.util.List;

@Service
public class ExternalClerkRelatedApiServiceImpl implements ExternalClerkRelatedApiService {

    private static final Logger log = LogManager.getLogger(ExternalClerkRelatedApiServiceImpl.class);

    @Autowired
    private TabHaobanExternalClerkRelatedMapper tabHaobanExternalClerkRelatedMapper;
    @Autowired
    private MemberUnionidRelatedMapper memberUnionidRelatedMapper;
    @Autowired
    private TabHaobanStaffClerkRelationMapper staffClerkRelationMapper;

    /**
     * 查询导购好友关联
     *
     * @param wxUserId       : 微信用户id
     * @param wxEnterpriseId : 微信企业id
     * @param enterpriseId   : 企业id
     * @return : 返回好友关联
     */
    @Override
    public List<ExternalClerkRelatedDTO> listByWxUserIdAndWxEnterpriseId(String wxUserId, String wxEnterpriseId, String enterpriseId) {
        List<TabHaobanExternalClerkRelated> list = tabHaobanExternalClerkRelatedMapper.listByWxUserIdAndWxEnterpriseId(wxUserId, wxEnterpriseId, enterpriseId);
        return EntityUtil.changeEntityListByJSON(ExternalClerkRelatedDTO.class, list);
    }

    /**
     * 查询会员在企业下的好友关系
     *
     * @param memberId       : 会员id
     * @param wxEnterpriseId : 微信企业id
     * @param enterpriseId   : 企业id
     * @return : 返回会员导购关联关系
     */
    @Override
    public List<ExternalClerkRelatedDTO> listByMemberIdAndWxEnterpriseId(String memberId, String wxEnterpriseId, String enterpriseId) {
        List<TabHaobanExternalClerkRelated> list = tabHaobanExternalClerkRelatedMapper.listByMemberIdAndWxEnterpriseId(memberId, wxEnterpriseId, enterpriseId);
        return EntityUtil.changeEntityListByJSON(ExternalClerkRelatedDTO.class, list);
    }

    @Override
    public List<ExternalClerkRelatedDTO> listByEnterpriseIdAndDate(
            String enterpriseId, Date date) {
        List<TabHaobanExternalClerkRelated> list = tabHaobanExternalClerkRelatedMapper.listByEnterpriseIdAndDate(enterpriseId, date);
        return EntityUtil.changeEntityListByJSON(ExternalClerkRelatedDTO.class, list);
    }

    @Override
    public Page<ExternalClerkRelatedDTO> pageMemberUnionByParams(List<String> userIdList, List<String> sendMemberIds,
                                                                 String enterpriseId, BasePageInfo pageInfo) {
        PageHelper.startPage(pageInfo.getPageNum(), pageInfo.getPageSize());
        return PageUtil.changePageHelperToCurrentPage(tabHaobanExternalClerkRelatedMapper.pageMemberUnionByParams(userIdList, sendMemberIds, enterpriseId), ExternalClerkRelatedDTO.class);
    }

    @Override
    public List<ExternalClerkRelatedDTO> listByWxEnterpriseIdAndWxUserId(String wxUserId, String wxEnterpriseId) {

        return EntityUtil.changeEntityListByJSON(ExternalClerkRelatedDTO.class, tabHaobanExternalClerkRelatedMapper.listByWxEnterpriseIdAndWxUserId(wxUserId, wxEnterpriseId));
    }

    @Override
    public List<ExternalClerkRelatedDTO> listByWxEnterpriseIdAndWxUserIds(List<String> userIdList,
                                                                          String wxEnterpriseId) {
        if (userIdList == null || userIdList.isEmpty()) {
            return Collections.EMPTY_LIST;
        }
        return EntityUtil.changeEntityListByJSON(ExternalClerkRelatedDTO.class, tabHaobanExternalClerkRelatedMapper.listByWxEnterpriseIdAndWxUserIds(userIdList, wxEnterpriseId));
    }

    @Override
    public ExternalClerkRelatedDTO getExternalClerkByRelationKeyAndPre(String relationKey) {
        TabHaobanExternalClerkRelated related = tabHaobanExternalClerkRelatedMapper.getByRelationKey(relationKey);
        return EntityUtil.changeEntityNew(ExternalClerkRelatedDTO.class, related);
    }

    @Override
    public Page<ExternalClerkRelatedDTO> pageExternalClerk(String wxEnterpriseId, String wxUserId, List<Integer> statusFlags, BasePageInfo pageInfo) {
        PageHelper.startPage(pageInfo);
        List<TabHaobanExternalClerkRelated> ret = tabHaobanExternalClerkRelatedMapper.listByWxEnterpriseIdAndWxUserIdAndStatus(wxEnterpriseId, wxUserId, statusFlags);
        Page<ExternalClerkRelatedDTO> retPage = PageUtil.changePageHelperToCurrentPage(new PageInfo<>(ret), ExternalClerkRelatedDTO.class);
        return retPage;
    }

    @Override
    public boolean pushExternalByRelationKey(String relationKey) {
        tabHaobanExternalClerkRelatedMapper.pushExternalByRelationKey(relationKey);
        return true;
    }

    @Override
    public ServiceResponse<ExternalClerkRelatedDTO> bindMemberIdExternalByRelationKey(String relationKey, String enterpriseId, String memberId, String unionid) {
        ServiceResponse<ExternalClerkRelatedDTO> resp = new ServiceResponse<>();
        TabHaobanExternalClerkRelated externalClerkRelated = tabHaobanExternalClerkRelatedMapper.getByRelationKey(relationKey);
        if (null == externalClerkRelated) {
            resp.setCode(0);
            resp.setMessage("relationKey对应的关联关系不存在或删除");
            return resp;
        }
        if (!externalClerkRelated.getEnterpriseId().equals(enterpriseId)) {
            resp.setCode(0);
            resp.setMessage("relationKey对应的企业与所传会员企业不对应");
            return resp;
        }
        externalClerkRelated.setMemberId(memberId);
        externalClerkRelated.setUnionid(unionid);
        externalClerkRelated.setUpdateTime(new Date());
        externalClerkRelated.setStatusFlag(1);
        tabHaobanExternalClerkRelatedMapper.updateByPrimaryKeySelective(externalClerkRelated);
        //更新unionid
        MemberUnionidRelated memberUnionidRelated = memberUnionidRelatedMapper.selectByPrimaryKey(externalClerkRelated.getMemberUnionidRelatedId());
        memberUnionidRelated.setUnionid(unionid);
        memberUnionidRelated.setUpdateTime(new Date());
        memberUnionidRelatedMapper.updateByPrimaryKeySelective(memberUnionidRelated);

        resp.setResult(EntityUtil.changeEntityNew(ExternalClerkRelatedDTO.class, externalClerkRelated));
        pushTagSync(memberId, externalClerkRelated.getWxEnterpriseId(), enterpriseId);
        return resp;
    }

    /**
     * 刷新标签信息
     *
     * @param memberId
     * @param wxEnterpriseId
     * @param enterpriseId
     */
    private void pushTagSync(String memberId, String wxEnterpriseId, String enterpriseId) {
        QywxTagSyncInfoPojo pojo = new QywxTagSyncInfoPojo();
        pojo.setDataId(memberId);
        pojo.setTaskId("-1");
        pojo.setTaskType(QywxSyncTaskTypeEnum.MEMBER_SYNC.getType());
        //单个会员同步
        pojo.setSyncType(GicQywxSyncTypeEnum.SINGLE_WXENT_MEMNER_SYNC.getType());
        pojo.setWxEnterpriseId(wxEnterpriseId);
        pojo.setEnterpriseId(enterpriseId);
        GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
        try {
            clientInstance.sendMessage("qywxTagSyncDeal", JSONObject.toJSONString(pojo));
        } catch (Exception e) {
            e.printStackTrace();
            log.info("异常：{}", e);
        }
    }

    @Override
    public ExternalClerkRelatedDTO getClerkFriendByUnionid(String enterpriseId, String clerkId, String unionid) {
        TabHaobanExternalClerkRelated externalClerkRelated = tabHaobanExternalClerkRelatedMapper.getClerkExternalByUnionId(enterpriseId, clerkId, unionid);
        ExternalClerkRelatedDTO clerkRelatedDTO = EntityUtil.changeEntityNew(ExternalClerkRelatedDTO.class, externalClerkRelated);
        return clerkRelatedDTO;
    }

    @Override
    public List<ExternalClerkRelatedDTO> listExternalClerkByWxUserIdAncExternalUserId(String wxEnterpriseId, String wxUserId, String enterpriseId, String externalUserId) {
        List<TabHaobanExternalClerkRelated> list = tabHaobanExternalClerkRelatedMapper.listExternalClerkByWxUserIdAncExternalUserId(wxEnterpriseId, wxUserId, enterpriseId, externalUserId);
        return EntityUtil.changeEntityListNew(ExternalClerkRelatedDTO.class, list);
    }

    @Override
    public List<ExternalClerkRelatedDTO> listByIds(String wxEnterpriseId, List<String> ids) {
        List<TabHaobanExternalClerkRelated> list = tabHaobanExternalClerkRelatedMapper.listByIds(wxEnterpriseId, ids);
        return EntityUtil.changeEntityListNew(ExternalClerkRelatedDTO.class, list);
    }

    @Override
    public Page<String> pageExternalByEnterpriseId(String wxEnterpriseId, String enterpriseId, BasePageInfo pageInfo) {
        PageHelper.startPage(pageInfo);
        List<String> memberIDs = tabHaobanExternalClerkRelatedMapper.listMemberIdByEnterpriseId(wxEnterpriseId, enterpriseId);
        return PageUtil.changePageHelperToCurrentPage(new PageInfo<>(memberIDs));
    }

    @Override
    public Page<String> pageExternalByEnterpriseId(String wxEnterpriseId, String enterpriseId, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<String> memberIDs = tabHaobanExternalClerkRelatedMapper.listMemberIdByEnterpriseId(wxEnterpriseId, enterpriseId);
        return PageUtil.changePageHelperToCurrentPage(new PageInfo<>(memberIDs));
    }

    @Override
    public List<String> listExternalClerkMemberIdsByMemberIds(String wxEnterpriseId, List<String> memberIds) {
        return tabHaobanExternalClerkRelatedMapper.listMemberIdsByMemberIds(memberIds, wxEnterpriseId);
    }

    @Override
    public List<ExternalClerkRelatedShortInfoDTO> listExternalClerkByMemberIds(String wxEnterpriseId, List<String> memberIds) {
        List<TabHaobanExternalClerkRelated> list = tabHaobanExternalClerkRelatedMapper.listByMemberIds(memberIds, wxEnterpriseId);
        return EntityUtil.changeEntityListNew(ExternalClerkRelatedShortInfoDTO.class, list);
    }

    @Override
    public List<ExternalClerkRelatedShortInfoDTO> listExternalClerkByMemberId(String memberId) {
        List<TabHaobanExternalClerkRelated> list = tabHaobanExternalClerkRelatedMapper.listByMemberId(memberId);
        return EntityUtil.changeEntityListNew(ExternalClerkRelatedShortInfoDTO.class, list);
    }

    @Override
    public void delByMemberIdAndEnterpriseId(String memberId, String enterpriseId, String mergeMemberId) {
        if (StringUtils.isBlank(mergeMemberId)) {
            //删除会员
            tabHaobanExternalClerkRelatedMapper.updateStatusByMemberId(memberId, enterpriseId);
        } else {
            //合并会员
            tabHaobanExternalClerkRelatedMapper.updateMemberIdByMemberId(mergeMemberId, memberId);
        }
    }

    @Override
    public boolean isFriends(String memberId, String clerkId, String enterpriseId) {
        TabHaobanStaffClerkRelation staffClerkRelation = staffClerkRelationMapper.getByClerkId(clerkId);
        if (staffClerkRelation == null) {
            return false;
        }
        String staffId = staffClerkRelation.getStaffId();
        log.info("【查询会员】clerkId = {},staffId={},memberId={}", clerkId, staffId, memberId);
        TabHaobanExternalClerkRelated externalClerkRelated = tabHaobanExternalClerkRelatedMapper.getByMemberIdAndStaffId(staffId, enterpriseId, memberId);
        if (externalClerkRelated == null) {
            return false;
        }
        return externalClerkRelated.getStatusFlag() == 1;
    }

}
